/* Generic Windows Process Patcher. Intended for patching nedmalloc in to
replace the MSVCRT allocator but could be used for anything.
(C) 2009 Niall Douglas

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#undef ERROR	/* Defined by windows */

#if defined(__cplusplus)
#if !defined(NO_WINPATCHER_NAMESPACE)
namespace winpatcher {
#else
extern "C" {
#endif
#endif

typedef enum StatusCode_t
{
	ERROR=-1,
	SUCCESS=0
} StatusCode;
typedef struct Status_t
{
	StatusCode code;				/* <0 = error, >=0 success */
	const char *sourcefile;
	int sourcelineno;
	TCHAR msg[65];
} Status;
#define MKSTATUS(ret, codev) ( ret.code=codev, ret.sourcefile=__FILE__, ret.sourcelineno=__LINE__, ret )
#define MKSTATUSWIN(ret) MakeStatusFromWindowsError(&ret, __FILE__, __LINE__)

static Status MakeStatusFromWindowsError(Status *ret, const char *file, int lineno)
{
	ret->code=-(int)GetLastError();
	ret->sourcefile=file;
	ret->sourcelineno=lineno;
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS, NULL, -ret->code, 0, ret->msg, (sizeof(ret->msg)/sizeof(TCHAR))-1, NULL);
	ret->msg[64]=0;
	return *ret;
}

static void MakeReportFromStatus(TCHAR *buffer, size_t bufflen, const Status *ret)
{
	TCHAR *p;
	const char *s;
	if(ret->code>=0)
		_stprintf_s(buffer, bufflen, __T("Success code %d at "), ret->code);
	else
		_stprintf_s(buffer, bufflen, __T("Error code %d (%s) at "), ret->code, ret->msg);
	for(p=_tcschr(buffer, 0), s=ret->sourcefile; *s; p++, s++)
		*p=*s;
	_stprintf_s(p, bufflen-(p-buffer)/sizeof(TCHAR), __T(":%d\n"), ret->sourcelineno);
}

#if defined(__cplusplus)
}
#endif
